%% Oscillations
sdist = 0.1;
x = [-100*pi:sdist:100*pi];
sinx = sin(x);
cosx = cos(x);

% (almost) orthogonal
sinx*cosx'

% check that sines/cosines at integer frequencies are orthogonal
for i=1:5, Y(i,:) = sin(i*x); end;
Y*Y'  % off-diagonal elements are zero

% now let's hear it
sound(sinx);
% what's the frequency of this sound?
% We need to know the sampling frequency
help sound

Fs = 10000; % we assume 10000 samples per second
% one sine/cosine cycle happens in 2*pi or 360 degrees
% in terms of samples, one cycle therefore is
scyc = 2*pi/sdist; 

% the desired frequency of our tone is
myfreq = 440;

% In other words:
% I want that my 10000 (sfreq) samples contain 440 (myfreq) cycles

fac = scyc / Fs; % this needs to be multiplied to myfreq

sound( sin( myfreq*fac*x ), Fs )

% write a flexible function
% play A minor chord
my_sound([440 523 659 880], fac, Fs);
% What's this?
my_sound([784 880 698 349 523], fac, Fs);
% A4 is 440 Hz, http://www.phy.mtu.edu/~suits/notefreqs.html

% try
my_sound([5000, 6000, 7000], fac, Fs)
% what's happening here?

% make keyboard, whole tone steps
my_keyboard(fac, Fs)


%% FFT

x = [-100:0.01:100];
y = sin(100*x);
yfft = fft(y);

% try
plot(yfft);

% better 
plot( abs(yfft) );
% why two peaks?
% the first peak is at 1000/(2*pi); and the second one?

% What is the frequency scale of this spectrum?
% We don't know, because we have not specified a sampling frequency yet
% Let's assume it's 10 kHz
Fs = 10000;

% We've got 1001 samples, i.e. we can get 500 frequencies plus the mean
% The highest frequency we can get is Fs/2 (Nyquist theorem)
% The mean is frequency 0
% other frequencies increase in steps of (Fs/2)/500, up to (Fs/2)
fftx = [0:(Fs/2)/5000:(Fs/2)];

plot(fftx, abs(yfft(1:5001)) );
% What's the peak's value? 
[val, elem] = max(yfft(1:5001))
fftx(elem)

%% limits of Fourier Analysis
x = [-5:0.1:5];
Fs = 10;

y = sin(x);
plot(x,y);

yfft = fft(y);

fftx = [0:(Fs/2)/50:(Fs/2)]; % note: we changed number of samples

plot(fftx, abs(yfft(1:51)) );
% why does the spectrum look so blurred?

% make a hanning window
h = hanning(101);
plot(h)
% multiply with data
plot(h.*y')
% note: the edges are diminished

% Fourier transform of "hanninged" data
yffth = fft(h.*y');

plot(fftx, abs(yffth(1:51)) );
hold on
plot(fftx, abs(yfft(1:51)), 'r' );
% note: the peak is more focal


%% Filtering
% first steps to filtering
close 

x = [-5:0.1:5];
y = sin(100*x) + sin(50*x);
plot(x, y)

yfft = fft(y);
plot( abs(yfft) );

% let's high-pass filter the data
yfft([31:71]) = 0;
plot( abs(yfft) );

% apply inverse Fourier transform to reconstitute data
yi = ifft(yfft);  % note: the output are complex numbers

% It's the real part of yi that represents the data
plot(x, y, 'r', x, real(yi), 'b');

% Filtering "real" data
load handel
sound(y, Fs)

yfft = fft(y);
yfftf = yfft;

ly = length(y);

% cut out lower frequencies
yfftf(1:ly/8) = 0;
yfftf(end-ly/8:end) = 0;
yf = ifft(yfftf);
sound(real(yf), Fs)

% cut out higher frequencies
yfftf = yfft;
yfftf(ly/2-ly/3:ly/2+ly/3) = 0;
yf = ifft(yfftf);
sound(real(yf), Fs)

% Note:
% this was just a demo
% this kind of "naive" filtering should usually not be applied,
% since it only works properly under idealised assumptions

%% Proper filtering

% First define your filter
% you specify the "order",
% and the cut-off frequency as a fraction of half the sampling frequency

% high-pass filter
[Bh,Ah] = butter(10, 0.2, 'high');
yf = filtfilt(Bh, Ah, y);
sound(yf, Fs)

% low-pass filter
[Bl,Al] = butter(10, 0.3, 'low');
yf = filtfilt(Bl, Al, y);
sound(yf, Fs)

% band-pass = high-pass plus low-pass
% apply high-pass filter to low-pass filtered data
yfb = filtfilt(Bh, Ah, yf);
sound(yfb, Fs)

% What's the difference between these two signals:
plot(yfb(1:500))
hold on
plot(y(1:500), 'r')
